/* $Id: CPUMInternal.h $ */
/** @file
 * CPUM - Internal header file.
 */

/*
 * Copyright (C) 2006-2012 Oracle Corporation
 *
 * This file is part of VirtualBox Open Source Edition (OSE), as
 * available from http://www.virtualbox.org. This file is free software;
 * you can redistribute it and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software
 * Foundation, in version 2 as it comes in the "COPYING" file of the
 * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 */



#pragma D  depends_on library x86.d
#pragma D  depends_on library cpumctx.d




/** @defgroup grp_cpum_int   Internals
 * @ingroup grp_cpum
 * @internal
 * @{
 */

/** Flags and types for CPUM fault handlers
 * @{ */
/** Type: Load DS */
inline uint8_t CPUM_HANDLER_DS = 1;

/** Type: Load ES */
inline uint8_t CPUM_HANDLER_ES = 2;

/** Type: Load FS */
inline uint8_t CPUM_HANDLER_FS = 3;

/** Type: Load GS */
inline uint8_t CPUM_HANDLER_GS = 4;

/** Type: IRET */
inline uint8_t CPUM_HANDLER_IRET = 5;

/** Type mask. */
inline uint8_t CPUM_HANDLER_TYPEMASK = 0xff;

/** If set EBP points to the CPUMCTXCORE that's being used. */
inline uint32_t CPUM_HANDLER_CTXCORE_IN_EBP = 1U << 31;

/** @} */


/** Use flags (CPUM::fUseFlags).
 * (Don't forget to sync this with CPUMInternal.mac!)
 * @{ */
/** Used the FPU, SSE or such stuff. */
inline uint32_t CPUM_USED_FPU = 1U << 0;

/** Used the FPU, SSE or such stuff since last we were in REM.
 * REM syncing is clearing this, lazy FPU is setting it. */
inline uint32_t CPUM_USED_FPU_SINCE_REM = 1U << 1;

/** Host OS is using SYSENTER and we must NULL the CS. */
inline uint32_t CPUM_USE_SYSENTER = 1U << 2;

/** Host OS is using SYSENTER and we must NULL the CS. */
inline uint32_t CPUM_USE_SYSCALL = 1U << 3;

/** Debug registers are used by host and must be disabled. */
inline uint32_t CPUM_USE_DEBUG_REGS_HOST = 1U << 4;

/** Enabled use of debug registers in guest context. */
inline uint32_t CPUM_USE_DEBUG_REGS = 1U << 5;

/** The XMM state was manually restored. (AMD only) */
inline uint32_t CPUM_MANUAL_XMM_RESTORE = 1U << 6;

/** Sync the FPU state on entry (32->64 switcher only). */
inline uint32_t CPUM_SYNC_FPU_STATE = 1U << 7;

/** Sync the debug state on entry (32->64 switcher only). */
inline uint32_t CPUM_SYNC_DEBUG_STATE = 1U << 8;

/** Enabled use of hypervisor debug registers in guest context. */
inline uint32_t CPUM_USE_DEBUG_REGS_HYPER = 1U << 9;

/** @} */

/* Sanity check. */


/**
 * The saved host CPU state.
 *
 * @remark  The special VBOX_WITH_HYBRID_32BIT_KERNEL checks here are for the 10.4.x series
 *          of Mac OS X where the OS is essentially 32-bit but the cpu mode can be 64-bit.
 */
typedef struct CPUMHOSTCTX
{
    /** FPU state. (16-byte alignment)
     * @remark On x86, the format isn't necessarily X86FXSTATE (not important). */
    X86FXSTATE      fpu;

    /** General purpose register, selectors, flags and more
     * @{ */
    /** General purpose register ++
     * { */
    /*uint64_t        rax; - scratch*/
    uint64_t        rbx;
    /*uint64_t        rcx; - scratch*/
    /*uint64_t        rdx; - scratch*/
    uint64_t        rdi;
    uint64_t        rsi;
    uint64_t        rbp;
    uint64_t        rsp;
    /*uint64_t        r8; - scratch*/
    /*uint64_t        r9; - scratch*/
    uint64_t        r10;
    uint64_t        r11;
    uint64_t        r12;
    uint64_t        r13;
    uint64_t        r14;
    uint64_t        r15;
    /*uint64_t        rip; - scratch*/
    uint64_t        rflags;

    /** @} */

    /** Selector registers
     * @{ */
    RTSEL           ss;
    RTSEL           ssPadding;
    RTSEL           gs;
    RTSEL           gsPadding;
    RTSEL           fs;
    RTSEL           fsPadding;
    RTSEL           es;
    RTSEL           esPadding;
    RTSEL           ds;
    RTSEL           dsPadding;
    RTSEL           cs;
    RTSEL           csPadding;
    /** @} */


    /** Control registers.
     * @{ */
    uint64_t        cr0;
    /*uint64_t        cr2; - scratch*/
    uint64_t        cr3;
    uint64_t        cr4;
    uint64_t        cr8;
    /** @} */

    /** Debug registers.
     * @{ */
    uint64_t        dr0;
    uint64_t        dr1;
    uint64_t        dr2;
    uint64_t        dr3;
    uint64_t        dr6;
    uint64_t        dr7;
    /** @} */

    /** Global Descriptor Table register. */
    X86XDTR64       gdtr;
    uint16_t        gdtrPadding;
    /** Interrupt Descriptor Table register. */
    X86XDTR64       idtr;
    uint16_t        idtrPadding;
    /** The task register. */
    RTSEL           ldtr;
    RTSEL           ldtrPadding;
    /** The task register. */
    RTSEL           tr;
    RTSEL           trPadding;

    /** MSRs
     * @{ */
    CPUMSYSENTER    SysEnter;
    uint64_t        FSbase;
    uint64_t        GSbase;
    uint64_t        efer;
    /** @} */

    /* padding to get 32byte aligned size */
    uint8_t         auPadding[8+32];

} CPUMHOSTCTX;
/** Pointer to the saved host CPU state. */
typedef CPUMHOSTCTX *PCPUMHOSTCTX;


/**
 * CPUM Data (part of VM)
 */
typedef struct CPUM
{
    /** Offset from CPUM to CPUMCPU for the first CPU. */
    uint32_t                offCPUMCPU0;

    /** Use flags.
     * These flags indicates which CPU features the host uses.
     */
    uint32_t                fHostUseFlags;

    /** Host CPU Features - ECX */
    struct
    {
        /** edx part */
        X86CPUIDFEATEDX     edx;
        /** ecx part */
        X86CPUIDFEATECX     ecx;
    } CPUFeatures;
    /** Host extended CPU features. */
    struct
    {
        /** edx part */
        uint32_t            edx;
        /** ecx part */
        uint32_t            ecx;
    } CPUFeaturesExt;

    /** Host CPU manufacturer. */
    CPUMCPUVENDOR           enmHostCpuVendor;
    /** Guest CPU manufacturer. */
    CPUMCPUVENDOR           enmGuestCpuVendor;

    /** CR4 mask */
    struct
    {
        uint32_t            AndMask;
        uint32_t            OrMask;
    } CR4;

    /** Synthetic CPU type? */
    uint8_t                    fSyntheticCpu;
    /** The (more) portable CPUID level.  */
    uint8_t                 u8PortableCpuIdLevel;
    /** Indicates that a state restore is pending.
     * This is used to verify load order dependencies (PGM). */
    uint8_t                    fPendingRestore;
    uint8_t                 abPadding[64 == 64 ? 5 : 1];

    /** The standard set of CpuId leaves. */
    CPUMCPUID               aGuestCpuIdStd[6];
    /** The extended set of CpuId leaves. */
    CPUMCPUID               aGuestCpuIdExt[10];
    /** The centaur set of CpuId leaves. */
    CPUMCPUID               aGuestCpuIdCentaur[4];
    /** The hypervisor specific set of CpuId leaves. */
    CPUMCPUID               aGuestCpuIdHyper[4];
    /** The default set of CpuId leaves. */
    CPUMCPUID               GuestCpuIdDef;


    RTHCPTR                 pvApicBase;
    uint32_t                fApicDisVectors;
    uint8_t                 abPadding3[4];
} CPUM;
/** Pointer to the CPUM instance data residing in the shared VM structure. */
typedef CPUM *PCPUM;

/**
 * CPUM Data (part of VMCPU)
 */
typedef struct CPUMCPU
{
    /**
     * Hypervisor context.
     * Aligned on a 64-byte boundary.
     */
    CPUMCTX                 Hyper;

    /**
     * Saved host context. Only valid while inside GC.
     * Aligned on a 64-byte boundary.
     */
    CPUMHOSTCTX             Host;


    /**
     * Guest context.
     * Aligned on a 64-byte boundary.
     */
    CPUMCTX                 Guest;

    /**
     * Guest context - misc MSRs
     * Aligned on a 64-byte boundary.
     */
    CPUMCTXMSRS             GuestMsrs;

    /** Use flags.
     * These flags indicates both what is to be used and what has been used.
     */
    uint32_t                fUseFlags;

    /** Changed flags.
     * These flags indicates to REM (and others) which important guest
     * registers which has been changed since last time the flags were cleared.
     * See the CPUM_CHANGED_* defines for what we keep track of.
     */
    uint32_t                fChanged;

    /** Offset from CPUM to CPUMCPU. */
    uint32_t                offCPUM;

    /** Temporary storage for the return code of the function called in the
     * 32-64 switcher. */
    uint32_t                u32RetCode;

    /** Have we entered raw-mode? */
    uint8_t                    fRawEntered;
    /** Have we entered the recompiler? */
    uint8_t                    fRemEntered;

    /** Align the structure on a 64-byte boundary. */
    uint8_t                 abPadding2[64 - 16 - 2];
} CPUMCPU;
/** Pointer to the CPUMCPU instance data residing in the shared VMCPU structure. */
typedef CPUMCPU *PCPUMCPU;


/** @} */


